/*
	Ariad V0.0.0.8
	Ariad (AutoRun.Inf Access Denied) started as a minifilter blocking access to \autorun.inf on USB sticks
	This version also blocks access to several file types on removable media
	USB Sticks are identified by their FILE_REMOVABLE_MEDIA device characteristics.
  USB HDD will not be recognized as USB sticks, and as such, content is not blocked.

  To install: use ariad-manual.inf
    You'll have to start the driver manually (also after each boot):
    net start ariad

    If you want to start the driver a boot time, use ariad-boot.inf
    You'll be prompted to restart your machine. This is because ariad will start a boot time. If you don't want to reboot,
    no problem. Cancel the reboot prompt and start the driver manually: net start ariad
    Next boot, ariad will be running automatically
    I advice to use ariad-boot.inf only after you've worked with the manual start option and are satisfied that your system is stable

  To uninstall:
    sc stop ariad
  	sc delete ariad
  	del \%windowsdir%\system32\drivers\ariad.inf

  To stop the driver (for example to allow autorun.inf)
  	net stop ariad

	Based on WDK minifilter examples
	Source code put in public domain by Didier Stevens, no Copyright
	https://DidierStevens.com
	Use at your own risk. It's beta software running inside the kernel

	Shortcomings, or todo's ;-)
	- get minifilter altitude allocation from Microsoft

	History:
	2009/01/24: Start development based on WDK minifilter examples
	2009/01/25: continue
	2009/01/28: ScannerPreCreate
	2009/01/30: ScannerPreCreate, block CD-ROM
	2009/01/31: AriadPreSetInformation
	2009/02/01: continue
	2009/02/09: registry, instance context
	2009/02/10: AllowDrives, SETTINGS_READONLY, SETTINGS_NOFILEEXECUTE
	2009/02/11: Time-based exception
	2009/02/15: code review, refactoring
	2009/11/04: V0.0.0.7 Extended the list ausExtensionsToBlock
	2010/07/15: V0.0.0.8 Extended the list ausExtensionsToBlock with .LNK
	2010/07/17: V0.0.0.8 Added couple of DbgPrint statements, fixed logic bug
*/

#include <fltKernel.h>
#include <dontuse.h>
#include <suppress.h>
#include "ariad.h"

#pragma prefast(disable:__WARNING_ENCODE_MEMBER_FUNCTION_POINTER, "Not valid for kernel mode drivers")

ARIAD_DATA sAriadData;

const UNICODE_STRING usAutorunInf = RTL_CONSTANT_STRING(L"autorun.inf");
const UNICODE_STRING usParentDir = RTL_CONSTANT_STRING(L"\\");

const UNICODE_STRING ausExtensionsToBlock[] = {
	RTL_CONSTANT_STRING(L"BAT"),
	RTL_CONSTANT_STRING(L"CMD"),
	RTL_CONSTANT_STRING(L"COM"),
	RTL_CONSTANT_STRING(L"CPL"),
	RTL_CONSTANT_STRING(L"DLL"),
	RTL_CONSTANT_STRING(L"EXE"),
	RTL_CONSTANT_STRING(L"OCX"),
	RTL_CONSTANT_STRING(L"PIF"),
	RTL_CONSTANT_STRING(L"SCR"),
	RTL_CONSTANT_STRING(L"SYS"),
	RTL_CONSTANT_STRING(L"VB"),
	RTL_CONSTANT_STRING(L"VBE"),
	RTL_CONSTANT_STRING(L"VBS"),
	RTL_CONSTANT_STRING(L"WSF"),
	RTL_CONSTANT_STRING(L"WSH"),
	RTL_CONSTANT_STRING(L"LNK"),
	{0, 0, NULL}
};

//  Assign text sections for each routine.

#ifdef ALLOC_PRAGMA
	#pragma alloc_text(INIT, DriverEntry)
	#pragma alloc_text(PAGE, AriadUnload)
	#pragma alloc_text(PAGE, AriadInstanceSetup)
	#pragma alloc_text(PAGE, AriadPreCreate)
	#pragma alloc_text(PAGE, AriadPreSetInformation)
	#pragma alloc_text(PAGE, AriadContextCleanup)
	#pragma alloc_text(PAGE, AriadInstanceTeardownComplete)
#endif

NTSTATUS DriverEntry(__in PDRIVER_OBJECT pDO, __in PUNICODE_STRING pusRegistryPath)
{
	NTSTATUS iNTStatus;

	const FLT_OPERATION_REGISTRATION Callbacks[] = {
		{IRP_MJ_CREATE, 0, AriadPreCreate, NULL},
		{IRP_MJ_SET_INFORMATION, 0, AriadPreSetInformation, NULL},
		{IRP_MJ_OPERATION_END}
	};

	const FLT_CONTEXT_REGISTRATION contextRegistration[] = {
		{FLT_INSTANCE_CONTEXT, 0, AriadContextCleanup, ARIAD_INSTANCE_CONTEXT_SIZE, ARIAD_INSTANCE_CONTEXT_TAG},
		{FLT_CONTEXT_END}
	};

	const FLT_REGISTRATION FilterRegistration = {
		sizeof(FLT_REGISTRATION),         //  Size
		FLT_REGISTRATION_VERSION,           //  Version
		0,                                  //  Flags
		contextRegistration,                //  Context Registration.
		Callbacks,                          //  Operation callbacks
		AriadUnload,                      //  FilterUnload
		AriadInstanceSetup,               //  InstanceSetup
		NULL,                             //  InstanceQueryTeardown
		NULL,                             //  InstanceTeardownStart
		AriadInstanceTeardownComplete,      //  InstanceTeardownComplete
		NULL,                               //  GenerateFileName
		NULL,                               //  GenerateDestinationFileName
		NULL                                //  NormalizeNameComponent
	};

	sAriadData.hDriverRegKey = NULL;
	ReadRegistrySettings(pusRegistryPath);

	iNTStatus = FltRegisterFilter(pDO, &FilterRegistration, &sAriadData.Filter);

	if (!NT_SUCCESS(iNTStatus))
		return iNTStatus;

	iNTStatus = FltStartFiltering(sAriadData.Filter);

	if (NT_SUCCESS(iNTStatus))
		return STATUS_SUCCESS;

	FltUnregisterFilter(sAriadData.Filter);

	return iNTStatus;
}

NTSTATUS AriadUnload(__in FLT_FILTER_UNLOAD_FLAGS iFFUF)
{
	UNREFERENCED_PARAMETER(iFFUF);

	ZwClose(sAriadData.hDriverRegKey);

	FltUnregisterFilter(sAriadData.Filter);

	return STATUS_SUCCESS;
}

int VolumeToDosName(__in PFLT_VOLUME pFV)
{
	PDEVICE_OBJECT pDO = NULL;
	UNICODE_STRING usDosName;
	NTSTATUS iNTStatus;
	int iReturn = 0;

	iNTStatus = FltGetDiskDeviceObject(pFV, &pDO);
	if (NT_SUCCESS(iNTStatus))
	{
		iNTStatus = IoVolumeDeviceToDosName(pDO, &usDosName);
		if (NT_SUCCESS(iNTStatus))
		{
			iReturn = RtlUpcaseUnicodeChar(usDosName.Buffer[0]);
			ExFreePool(usDosName.Buffer);
		}
	}

	return iReturn;
}

NTSTATUS AriadInstanceSetup(__in PCFLT_RELATED_OBJECTS pFRO, __in FLT_INSTANCE_SETUP_FLAGS iFISF, __in DEVICE_TYPE iDT, __in FLT_FILESYSTEM_TYPE iFFT)
{
	UCHAR ucBuffer[sizeof(FLT_VOLUME_PROPERTIES)+512];
	PFLT_VOLUME_PROPERTIES pFVP = (PFLT_VOLUME_PROPERTIES)ucBuffer;
	ULONG ulLengthReceived;
	NTSTATUS iNTStatus;
	NTSTATUS iNTStatusReturn;
	PARIAD_INSTANCE_CONTEXT pAIC = NULL;
	int iDrive;
	int iIter;
	LARGE_INTEGER liSystemTime;
	__int64 iSystemTime;

	UNREFERENCED_PARAMETER(iFISF);
	UNREFERENCED_PARAMETER(iFFT);

	ulLengthReceived = 0;

	ReadRegistrySettings(NULL);

	PAGED_CODE();

	ASSERT(pFRO->Filter == sAriadData.Filter);

	KeQuerySystemTime(&liSystemTime);
	iSystemTime = liSystemTime.HighPart * 0x100000000 + liSystemTime.LowPart;
	if (sAriadData.iExceptionTime > iSystemTime)
		return STATUS_FLT_DO_NOT_ATTACH;

	iDrive = VolumeToDosName(pFRO->Volume);
	if (iDrive != 0)
		for (iIter = 0; iIter < 26; iIter++)
			if (sAriadData.aiAllowDrives[iIter] == iDrive)
				return STATUS_FLT_DO_NOT_ATTACH;

	iNTStatus = FltAllocateContext(pFRO->Filter, FLT_INSTANCE_CONTEXT, ARIAD_INSTANCE_CONTEXT_SIZE, NonPagedPool, &pAIC);

	if (!NT_SUCCESS(iNTStatus))
	{
		if (pAIC != NULL)
			FltReleaseContext(pAIC);
		return STATUS_FLT_DO_NOT_ATTACH;
	}

	// 3 FILE_DEVICE_CD_ROM_FILE_SYSTEM
	// 8 FILE_DEVICE_DISK_FILE_SYSTEM
	// 20 FILE_DEVICE_NETWORK_FILE_SYSTEM
	DbgPrint("Ariad: AriadInstanceSetup iDT: %d", iDT);
	switch (iDT)
	{
		case FILE_DEVICE_CD_ROM_FILE_SYSTEM:
			pAIC->ucFlags = (unsigned char)SETTINGS_CDROM(sAriadData.ulSettings);
			iNTStatusReturn = pAIC->ucFlags ? STATUS_SUCCESS : STATUS_FLT_DO_NOT_ATTACH;
			break;

		case FILE_DEVICE_DISK_FILE_SYSTEM:
			pAIC->ucFlags = (unsigned char)SETTINGS_DISK(sAriadData.ulSettings);
			iNTStatusReturn = pAIC->ucFlags ? STATUS_SUCCESS : STATUS_FLT_DO_NOT_ATTACH;
			if (NT_SUCCESS(FltGetVolumeProperties(pFRO->Volume, pFVP, sizeof(ucBuffer), &ulLengthReceived)))
			{
				DbgPrint("Ariad: AriadInstanceSetup FILE_DEVICE_DISK_FILE_SYSTEM: RealDeviceName: %wZ DeviceCharacteristics: %08x", &pFVP->RealDeviceName, pFVP->DeviceCharacteristics);
				if (pFVP->DeviceCharacteristics & FILE_REMOVABLE_MEDIA)
				{
					pAIC->ucFlags = (unsigned char)SETTINGS_REMOVABLE(sAriadData.ulSettings);
					iNTStatusReturn = pAIC->ucFlags ? STATUS_SUCCESS : STATUS_FLT_DO_NOT_ATTACH;
				}
			}
			break;

		case FILE_DEVICE_NETWORK_FILE_SYSTEM:
			pAIC->ucFlags = (unsigned char)SETTINGS_NETWORK(sAriadData.ulSettings);
			iNTStatusReturn = pAIC->ucFlags ? STATUS_SUCCESS : STATUS_FLT_DO_NOT_ATTACH;
			break;

		default:
			iNTStatusReturn = STATUS_FLT_DO_NOT_ATTACH;
			break;
	}
	DbgPrint("Ariad: AriadInstanceSetup iNTStatusReturn: %d", iNTStatusReturn);

	iNTStatus = FltSetInstanceContext(pFRO->Instance, FLT_SET_CONTEXT_KEEP_IF_EXISTS, pAIC, NULL);

	if (!NT_SUCCESS(iNTStatus))
	{
		if (pAIC != NULL)
			FltReleaseContext(pAIC);
		return STATUS_FLT_DO_NOT_ATTACH;
	}

	if (pAIC != NULL)
		FltReleaseContext(pAIC);

	if (NT_SUCCESS(iNTStatusReturn))
	{
		if (ulLengthReceived <= 0)
			FltGetVolumeProperties(pFRO->Volume, pFVP, sizeof(ucBuffer), &ulLengthReceived);
		DbgPrint("Ariad: Instance setup iDT: %d RealDeviceName: %wZ DeviceCharacteristics: %08x Dosname: %c", iDT, &pFVP->RealDeviceName, pFVP->DeviceCharacteristics, iDrive);
	}

	return iNTStatusReturn;
}

BOOLEAN MatchAutorunInf(PFLT_FILE_NAME_INFORMATION pFFNI)
{
	return RtlCompareUnicodeString(&usAutorunInf, &pFFNI->FinalComponent, TRUE) == 0 && RtlCompareUnicodeString(&usParentDir, &pFFNI->ParentDir, TRUE) == 0;
}

BOOLEAN CheckExtension(__in PUNICODE_STRING pusExtension)
{
	PUNICODE_STRING pusIterExtension;

	for (pusIterExtension = ausExtensionsToBlock; pusIterExtension->Buffer != NULL; pusIterExtension++)
		if (RtlCompareUnicodeString(pusExtension, pusIterExtension, TRUE) == 0)
			return TRUE;

	return FALSE;
}

FLT_PREOP_CALLBACK_STATUS AriadPreCreate(__inout PFLT_CALLBACK_DATA pFCD, __in PCFLT_RELATED_OBJECTS pFRO, __deref_out_opt PVOID *ppvCompletionContext)
{
	PFLT_FILE_NAME_INFORMATION pFFNI;
	NTSTATUS iNTStatus;
	BOOLEAN bMatchFound;
	PARIAD_INSTANCE_CONTEXT pAIC = NULL;
	unsigned char ucFlags;

	UNREFERENCED_PARAMETER(pFRO);
	UNREFERENCED_PARAMETER(ppvCompletionContext);

	PAGED_CODE();

	iNTStatus = FltGetInstanceContext(pFCD->Iopb->TargetInstance, &pAIC);
	if (!NT_SUCCESS(iNTStatus))
		return FLT_PREOP_SUCCESS_NO_CALLBACK;
	ucFlags = pAIC->ucFlags;
	if (pAIC != NULL)
		FltReleaseContext(pAIC);

	iNTStatus = FltGetFileNameInformation(pFCD, FLT_FILE_NAME_NORMALIZED | FLT_FILE_NAME_QUERY_DEFAULT, &pFFNI);
	if (!NT_SUCCESS(iNTStatus))
		return FLT_PREOP_SUCCESS_NO_CALLBACK;

	FltParseFileNameInformation(pFFNI);
	bMatchFound = FALSE;
	if (!bMatchFound && ucFlags & SETTINGS_AUTORUNINF)
		bMatchFound = MatchAutorunInf(pFFNI);
	if (!bMatchFound && ucFlags & SETTINGS_EXECUTABLES)
		bMatchFound = CheckExtension(&(pFFNI->Extension));
	if (!bMatchFound && ucFlags & SETTINGS_BLOCK)
		bMatchFound = TRUE;
	if (!bMatchFound && ucFlags & SETTINGS_READONLY)
		bMatchFound = (pFCD->Iopb->Parameters.Create.SecurityContext->DesiredAccess & (DELETE|FILE_WRITE_DATA|FILE_WRITE_ATTRIBUTES|FILE_WRITE_EA|FILE_APPEND_DATA|WRITE_DAC|WRITE_OWNER)) != 0;
	if (!bMatchFound && ucFlags & SETTINGS_NOFILEEXECUTE)
		bMatchFound = (pFCD->Iopb->Parameters.Create.SecurityContext->DesiredAccess & FILE_EXECUTE) != 0;
	FltReleaseFileNameInformation(pFFNI);

	if (!bMatchFound)
		return FLT_PREOP_SUCCESS_NO_CALLBACK;
	else
	{
		DbgPrint("Ariad pre Create: access denied");

		pFCD->IoStatus.Status = STATUS_ACCESS_DENIED;
		pFCD->IoStatus.Information = 0;

		return FLT_PREOP_COMPLETE;
	}
}

FLT_PREOP_CALLBACK_STATUS AriadPreSetInformation(__inout PFLT_CALLBACK_DATA pFCD, __in PCFLT_RELATED_OBJECTS pFRO, __deref_out_opt PVOID *ppvCompletionContext)
{
	PFILE_RENAME_INFORMATION pFRI;
	PFLT_FILE_NAME_INFORMATION pFFNI;
	BOOLEAN bMatchFound;

	UNREFERENCED_PARAMETER(ppvCompletionContext);

	PAGED_CODE();

	if (FileRenameInformation == pFCD->Iopb->Parameters.SetFileInformation.FileInformationClass)
	{
		pFRI = (FILE_RENAME_INFORMATION *)pFCD->Iopb->Parameters.SetFileInformation.InfoBuffer;
		FltGetDestinationFileNameInformation(pFRO->Instance, pFRO->FileObject, pFRI->RootDirectory, pFRI->FileName, pFRI->FileNameLength, FLT_FILE_NAME_NORMALIZED | FLT_FILE_NAME_QUERY_DEFAULT, &pFFNI);
		FltParseFileNameInformation(pFFNI);
		bMatchFound = MatchAutorunInf(pFFNI);
		FltReleaseFileNameInformation(pFFNI);

		if (bMatchFound)
		{
			DbgPrint("Ariad pre SetInformation: rename to \\autorun.inf access denied");

			pFCD->IoStatus.Status = STATUS_ACCESS_DENIED;
			pFCD->IoStatus.Information = 0;

			return FLT_PREOP_COMPLETE;
		}
	}

	return FLT_PREOP_SUCCESS_NO_CALLBACK;
}

VOID ReadRegistrySettings(__in PUNICODE_STRING pusRegistryPath)
{
	OBJECT_ATTRIBUTES sOA;
	NTSTATUS iNTStatus;
	ULONG ulResultLength;
	UNICODE_STRING usValueName;
//	UCHAR buffer[sizeof(KEY_VALUE_PARTIAL_INFORMATION) + sizeof(LONG)];
	UCHAR aucBuffer[sizeof(KEY_VALUE_PARTIAL_INFORMATION) + 1024];
	UNICODE_STRING usDrives;
	int iIter;

	if (NULL == sAriadData.hDriverRegKey)
	{
		InitializeObjectAttributes(&sOA, pusRegistryPath, OBJ_CASE_INSENSITIVE | OBJ_KERNEL_HANDLE, NULL, NULL);

		iNTStatus = ZwOpenKey(&sAriadData.hDriverRegKey, KEY_READ, &sOA);

		if (!NT_SUCCESS(iNTStatus))
		{
			sAriadData.hDriverRegKey = NULL;
			return;
		}
	}

	RtlInitUnicodeString(&usValueName, L"Settings");

	iNTStatus = ZwQueryValueKey(sAriadData.hDriverRegKey, &usValueName, KeyValuePartialInformation, aucBuffer, sizeof(aucBuffer), &ulResultLength);

	if (NT_SUCCESS(iNTStatus))
		sAriadData.ulSettings = *((PULONG) &(((PKEY_VALUE_PARTIAL_INFORMATION)aucBuffer)->Data));
	else
		sAriadData.ulSettings = 0x00000101;

	for (iIter = 0; iIter < 26; iIter++)
		sAriadData.aiAllowDrives[iIter] = 0;

	RtlInitUnicodeString(&usValueName, L"AllowDrives");

	iNTStatus = ZwQueryValueKey(sAriadData.hDriverRegKey, &usValueName, KeyValuePartialInformation, aucBuffer, sizeof(aucBuffer), &ulResultLength);

	if (NT_SUCCESS(iNTStatus))
	{
		RtlInitUnicodeString(&usDrives, ((PCWSTR) &(((PKEY_VALUE_PARTIAL_INFORMATION)aucBuffer)->Data)));
		for (iIter = 0; iIter < usDrives.Length && iIter < 26; iIter++)
			sAriadData.aiAllowDrives[iIter] = RtlUpcaseUnicodeChar(usDrives.Buffer[iIter]);
	}

	RtlInitUnicodeString(&usValueName, L"ExceptionTimeHighPart");

	iNTStatus = ZwQueryValueKey(sAriadData.hDriverRegKey, &usValueName, KeyValuePartialInformation, aucBuffer, sizeof(aucBuffer), &ulResultLength);

	if (NT_SUCCESS(iNTStatus))
		sAriadData.iExceptionTime = *((PULONG) &(((PKEY_VALUE_PARTIAL_INFORMATION)aucBuffer)->Data)) * 0x100000000;
	else
		sAriadData.iExceptionTime = 0;

	RtlInitUnicodeString(&usValueName, L"ExceptionTimeLowPart");

	iNTStatus = ZwQueryValueKey(sAriadData.hDriverRegKey, &usValueName, KeyValuePartialInformation, aucBuffer, sizeof(aucBuffer), &ulResultLength);

	if (NT_SUCCESS(iNTStatus))
		sAriadData.iExceptionTime += *((PULONG) &(((PKEY_VALUE_PARTIAL_INFORMATION)aucBuffer)->Data));
}

VOID AriadContextCleanup(__in PFLT_CONTEXT pFC, __in FLT_CONTEXT_TYPE pFCT)
{
	PARIAD_INSTANCE_CONTEXT pAIC;

	PAGED_CODE();

	if (FLT_INSTANCE_CONTEXT == pFCT)
	{
		pAIC = (PARIAD_INSTANCE_CONTEXT) pFC;
		// no memory to release
	}
}

VOID AriadInstanceTeardownComplete(__in PCFLT_RELATED_OBJECTS pFRO, __in FLT_INSTANCE_TEARDOWN_FLAGS iFITF)
{
	PARIAD_INSTANCE_CONTEXT pAIC;
	NTSTATUS iNTStatus;

	UNREFERENCED_PARAMETER(iFITF);

	PAGED_CODE();

	iNTStatus = FltGetInstanceContext(pFRO->Instance, &pAIC);
	if (NT_SUCCESS(iNTStatus))
		FltReleaseContext(pAIC);
}
