/*
	Ariad
	Source code put in public domain by Didier Stevens, no Copyright
	http://didierstevens.com
	Use at your own risk

	Shortcommings, or todo's ;-)
	
	History:
		2009/02/11: start
		2009/09/28: 30 seconds
		2009/11/03: v0.0.0.7 added dialog
		2009/11/04: ..\common\ariad.h
*/

#include <stdio.h>
#include <windows.h>
#include <tchar.h>
#include "..\common\ariad.h"
#include "resource.h"

#ifdef INCLUDE_RESOURCE
#pragma resource "ariad.res"
#endif

#define IS_SUCCESS(x) (ERROR_SUCCESS == (x))

#define SECONDS_TO_DISABLE 60

DWORD ReadSettingsFromRegistry(void);
void WriteSettingsToRegistry(DWORD dwSettings);
LPTSTR ReadAllowDrivesFromRegistry(void);
void WriteAllowDrivesToRegistry(LPTSTR szAllowDrives);

DWORD ReadSettingsFromRegistry(void)
{
	HKEY hKey;
	DWORD dwSettings;
	DWORD dwSize;
	DWORD dwType;
		
	if (IS_SUCCESS(RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGISTRY_PATH, 0L, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL)))
	{
		dwSize = sizeof(dwSettings);
		if (!IS_SUCCESS(RegQueryValueEx(hKey, REGISTRY_SETTINGS, NULL, &dwType, (LPBYTE) &dwSettings, &dwSize)))
		{
			dwSettings = 0x0101;
			WriteSettingsToRegistry(dwSettings);
		}
		RegCloseKey(hKey);
	}
	
	return dwSettings;
}

void WriteSettingsToRegistry(DWORD dwSettings)
{
	HKEY hKey;
		
	if (IS_SUCCESS(RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGISTRY_PATH, 0L, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL)))
	{
		if (!IS_SUCCESS(RegSetValueEx(hKey, REGISTRY_SETTINGS, NULL, REG_DWORD, (CONST LPBYTE) &dwSettings, sizeof(dwSettings))))
			MessageBox(NULL, _TEXT("Error writing to registry"), _TEXT("Ariad"), MB_OK | MB_ICONEXCLAMATION);
		RegCloseKey(hKey);
	}
	
	return;
}

LPTSTR ReadAllowDrivesFromRegistry(void)
{
	HKEY hKey;
	DWORD dwSize;
	DWORD dwType;
	static TCHAR szAllowDrives[256];
		
	if (IS_SUCCESS(RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGISTRY_PATH, 0L, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL)))
	{
		dwSize = sizeof(szAllowDrives) / sizeof(TCHAR);
		if (!IS_SUCCESS(RegQueryValueEx(hKey, REGISTRY_ALLOWDRIVES, NULL, &dwType, (LPBYTE) &szAllowDrives, &dwSize)))
		{
			_tcscpy(szAllowDrives, _TEXT(""));
			WriteAllowDrivesToRegistry(szAllowDrives);
		}
		RegCloseKey(hKey);
	}
	
	return szAllowDrives;
}

void WriteAllowDrivesToRegistry(LPTSTR szAllowDrives)
{
	HKEY hKey;
		
	if (IS_SUCCESS(RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGISTRY_PATH, 0L, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL)))
	{
		if (!IS_SUCCESS(RegSetValueEx(hKey, REGISTRY_ALLOWDRIVES, NULL, REG_SZ, (CONST LPBYTE) szAllowDrives, strlen(szAllowDrives) + 1)))
			MessageBox(NULL, _TEXT("Error writing to registry"), _TEXT("Ariad"), MB_OK | MB_ICONEXCLAMATION);
		RegCloseKey(hKey);
	}
	
	return;
}

void DisableAriadForNewDrivesDuringXSeconds(int iSecondsToDisable)
{
	HKEY hKey;
	FILETIME ftNow;
	__int64 iNowPlusXSeconds;
		
	if (IS_SUCCESS(RegCreateKeyEx(HKEY_LOCAL_MACHINE, REGISTRY_PATH, 0L, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL)))
	{
		GetSystemTimeAsFileTime(&ftNow);
		iNowPlusXSeconds = ftNow.dwHighDateTime * 0x100000000 + ftNow.dwLowDateTime + iSecondsToDisable * 10000000;
		ftNow.dwHighDateTime = iNowPlusXSeconds / 0x100000000;
		ftNow.dwLowDateTime = iNowPlusXSeconds & 0xFFFFFFFF;
		RegSetValueEx(hKey, REGISTRY_EXCEPTIONTIMEHIGHPART, 0L, REG_DWORD, (CONST LPBYTE) &ftNow.dwHighDateTime, sizeof(ftNow.dwHighDateTime));
		RegSetValueEx(hKey, REGISTRY_EXCEPTIONTIMELOWPART, 0L, REG_DWORD, (CONST LPBYTE) &ftNow.dwLowDateTime, sizeof(ftNow.dwLowDateTime));
		RegCloseKey(hKey);
	}
}

void SetCheckBoxes(HWND hwnd, DWORD dwSettings)
{
	SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_BLOCK_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_BLOCK_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_BLOCK_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_BLOCK_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_READONLY_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_READONLY_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_READONLY_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_READONLY_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
	SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
}

#ifdef __BORLANDC__
#pragma warn -8057
#endif

BOOL CALLBACK DlgProc(HWND hwnd, UINT uiMessage, WPARAM wParam, LPARAM lParam)
{
	DWORD dwSettings;
	
	switch(uiMessage)
	{
		case WM_INITDIALOG:
			SendMessage(hwnd, WM_SETICON, ICON_SMALL, (LPARAM)LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MY_ICON)));
			SendMessage(hwnd, WM_SETICON, ICON_BIG, (LPARAM)LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MY_ICON)));
			SetCheckBoxes(hwnd, ReadSettingsFromRegistry());
			SetDlgItemText(hwnd, ID_ED_ALLOWDRIVES, ReadAllowDrivesFromRegistry());
			break;
		case WM_CLOSE:
			EndDialog(hwnd, 0);
			break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDOK:
					EndDialog(hwnd, 0);
					break;
				case ID_DISABLE:
					DisableAriadForNewDrivesDuringXSeconds(SECONDS_TO_DISABLE);
					MessageBox(hwnd, _TEXT("Disabled for 60 seconds"), _TEXT("Ariad"), MB_OK | MB_ICONEXCLAMATION);
					break;
				case ID_CB_AUTORUNINF_REMOVABLE:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_REMOVABLE(SETTINGS_AUTORUNINF);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_AUTORUNINF_CDROM:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_CDROM(SETTINGS_AUTORUNINF);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_AUTORUNINF_DISK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_DISK(SETTINGS_AUTORUNINF);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_AUTORUNINF_NETWORK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_NETWORK(SETTINGS_AUTORUNINF);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_AUTORUNINF_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_AUTORUNINF ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_EXECUTABLES_REMOVABLE:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_REMOVABLE(SETTINGS_EXECUTABLES);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_EXECUTABLES_CDROM:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_CDROM(SETTINGS_EXECUTABLES);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_EXECUTABLES_DISK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_DISK(SETTINGS_EXECUTABLES);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_EXECUTABLES_NETWORK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_NETWORK(SETTINGS_EXECUTABLES);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_EXECUTABLES_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_EXECUTABLES ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_BLOCK_REMOVABLE:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_REMOVABLE(SETTINGS_BLOCK);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_BLOCK_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_BLOCK_CDROM:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_CDROM(SETTINGS_BLOCK);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_BLOCK_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_BLOCK_DISK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_DISK(SETTINGS_BLOCK);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_BLOCK_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_BLOCK_NETWORK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_NETWORK(SETTINGS_BLOCK);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_BLOCK_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_BLOCK ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_READONLY_REMOVABLE:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_REMOVABLE(SETTINGS_READONLY);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_READONLY_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_READONLY_CDROM:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_CDROM(SETTINGS_READONLY);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_READONLY_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_READONLY_DISK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_DISK(SETTINGS_READONLY);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_READONLY_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_READONLY_NETWORK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_NETWORK(SETTINGS_READONLY);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_READONLY_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_READONLY ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_NOFILEEXECUTE_REMOVABLE:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_REMOVABLE(SETTINGS_NOFILEEXECUTE);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_REMOVABLE, BM_SETCHECK, SETTINGS_REMOVABLE(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_NOFILEEXECUTE_CDROM:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_CDROM(SETTINGS_NOFILEEXECUTE);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_CDROM, BM_SETCHECK, SETTINGS_CDROM(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_NOFILEEXECUTE_DISK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_DISK(SETTINGS_NOFILEEXECUTE);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_DISK, BM_SETCHECK, SETTINGS_DISK(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_CB_NOFILEEXECUTE_NETWORK:
					dwSettings = ReadSettingsFromRegistry() ^ SETTINGS_SET_NETWORK(SETTINGS_NOFILEEXECUTE);
					WriteSettingsToRegistry(dwSettings);
					SendDlgItemMessage(hwnd, ID_CB_NOFILEEXECUTE_NETWORK, BM_SETCHECK, SETTINGS_NETWORK(dwSettings) & SETTINGS_NOFILEEXECUTE ? BST_CHECKED : BST_UNCHECKED, 0);
					break;
				case ID_ED_ALLOWDRIVES:
					switch(HIWORD(wParam))
					{
						TCHAR szAllowDrives[256];
						case EN_CHANGE:
							GetDlgItemText(hwnd, ID_ED_ALLOWDRIVES, szAllowDrives, sizeof(szAllowDrives) / sizeof(TCHAR));
							WriteAllowDrivesToRegistry(szAllowDrives);
							break;
					}
					break;
			}
			break;
		case WM_DESTROY:
			break;
		default:
			return FALSE;
	}
	return TRUE;
}

#ifdef __BORLANDC__
#pragma warn +8057
#endif

#ifdef __BORLANDC__
#pragma warn -8057
#endif

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	return DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, DlgProc);
}

#ifdef __BORLANDC__
#pragma warn +8057
#endif
